@@ -2,7 +2,7 @@
 |   Copyright (C) 1999-2000  Jochen C. Loewer (loewerj@hotmail.com)
 +----------------------------------------------------------------------------
 |
-|   $Header: /usr/local/pubcvs/tdom/generic/domalloc.c,v 1.7 2003/03/06 13:58:53 zoran Exp $
+|   $Id$
 |
 |
 |   A special memory allocator, which uses pre-allocated / bit masked
@@ -34,7 +34,6 @@
 |
 \--------------------------------------------------------------------------*/
 
-
 /*---------------------------------------------------------------------------
 |   Includes
 |
@@ -147,8 +146,8 @@ fillHashTable (
             /* all is fine, block is already in hash table */
             return;
         }
-        if      (hashedBlock->hashIndex1 == i) hashedBlock = hashedBlock->hashNext1;
-        else if (hashedBlock->hashIndex2 == i) hashedBlock = hashedBlock->hashNext2;
+        if      (hashedBlock->hashIndex1 == (int)i) hashedBlock = hashedBlock->hashNext1;
+        else if (hashedBlock->hashIndex2 == (int)i) hashedBlock = hashedBlock->hashNext2;
         else hashedBlock = NULL;
     }
 
@@ -577,4 +576,3 @@ domFree (
     }
     TDomThreaded(Tcl_MutexUnlock(&binMutex);) /* UNLOCK !*/
 }
-
