@@ -16,18 +16,23 @@ if {$tcl_version >= 8.1} {
             ::tcltest::testConstraint] == -1} {
         set ::tcltest::testConfig(need_i18n) 1
         set ::tcltest::testConstraints(need_i18n) 1
-        if {[info procs ::tDOM::extRefHandler] != ""} {
+        if {[info procs ::tdom::extRefHandler] != ""} {
             set ::tcltest::testConfig(need_uri) 1
             set ::tcltest::testConstraints(need_uri) 1
         }
     } else {
         ::tcltest::testConstraint need_i18n 1
-        if {[info procs ::tDOM::extRefHandler] != ""} {
+        if {[info procs ::tdom::extRefHandler] != ""} {
             ::tcltest::testConstraint need_uri 1
         }
     }
 }
 
+::tcltest::testConstraint needExpand 1
+if {$tcl_version < 8.5} {
+    ::tcltest::testConstraint needExpand 0
+}
+    
 set timeCmd {clock format [clock seconds]}
 
 set ::tcltest::testSingleFile false
